//
// AnalogInput_InternalThermometer_430
// Analog Input - MSP430 Internal Thermometer
//
// Demonstrates analog input by reading the internal temperature sensor.
//
// Created by Robert Wessels
// modified 7 March 2012
// By Robert Wessels
//
// This example code is in the public domain.
//
// Revision:
// Rei VILO, Mar 12, 2012 - One decimal place
// Rei VILO, Mar 13, 2012 - More precise algorithm
// Rei VILO, Mar 14, 2012 - Average
// Rei VILO, May 21, 2012 - Updated with GREEN_LED, RED_LED and PUSH2
// Rei VILO, Sep 09, 2012 - TimerSerial.h no longer required
// Rei VILO, Sep 24, 2012 - FraunchPad excluded
// Press push 2 to end
// Tested on msp430g2452 and msp430g2553
// 2196 bytes
#if defined(__MSP430G2452__) || defined(__MSP430G2553__) || defined(__MSP430G2231__)
#else
#error Board not supported
#endif
// RED_LED, GREEN_LED, TEMPSENSOR, PUSH2 already defined
#define NUMBER 32 // take number / 2
int ledState = HIGH;
uint8_t i = 0;
uint32_t average = 0;
uint32_t values[NUMBER];
uint8_t j = 0;
boolean flag = false;
boolean running = false;
int debounceCounter = 0;
void setup() {
  pinMode(RED_LED, OUTPUT);
  pinMode(GREEN_LED, OUTPUT);
  analogReference(INTERNAL1V5);
  analogRead(TEMPSENSOR); // first reading usually wrong
  Serial.begin(9600);
  pinMode(PUSH2, INPUT_PULLUP);
  digitalWrite(RED_LED, HIGH);
  digitalWrite(GREEN_LED, LOW);
  // Serial.print("\n\n\n*** MSP430 Thermometer \n");
  // Serial.print("Press PUSH2 to end\n");
  // Serial.print("instant\taverage\n");
  for (j=0; j<NUMBER; j++) {
    values[j]=0;
  }
  average = 0;
  j=0;
}

void printDec(uint32_t ui) {
  if (ui < 100){
    // ensure that a leading zero is printed so that 
    // all temperatures will have 4 digits
    Serial.print("0");
  }
  if (ui < 999){
    Serial.print(ui/10, DEC);
    Serial.print(".");
    Serial.print(ui%10, DEC);
  } 
  else {
    Serial.print("99.9");
  }
}

void loop() {
  if (running){
    ledState = !ledState;
    // LEDs: green = ready; red = acquisition
    digitalWrite(RED_LED, ledState);
//    digitalWrite(flag ? GREEN_LED : RED_LED, ledState);

    // Formula: http://www.43oh.com/forum/viewtopic.php?p=18248#p18248
    average -= values[j];
    // In Degrees Celsius:
    //values[j] = ((uint32_t)analogRead(TEMPSENSOR)*27069 - 18169625) *10 >> 16;
    // In Farenheit:
    values[j] = ((uint32_t)analogRead(TEMPSENSOR)*48724 - 30634388) *10 >> 16;
    average += values[j];
    // Print measure
    //printDec(values[j]);
    //Serial.print("\t");
    // Print average
    if (flag) {

      printDec(average/NUMBER);
      Serial.print("\n");
      //flag = false;

    }
    j++;
    if (j==NUMBER) {
      flag=true;
      j = 0;
    } 
  } 
  if (digitalRead(PUSH2)==LOW) {
    if (debounceCounter == 2){
      running = !running;
      digitalWrite(GREEN_LED, running);
      digitalWrite(RED_LED, HIGH);
      debounceCounter = 9;
    } else {
      debounceCounter++;
    }
  } else {
    debounceCounter = 0;
  }
  delay(40);

}



