/*
  Graph

  A simple example of communication from the Launchpad to the computer:
  the value of analog input 0 is sent out the serial port. We call this "serial"
  communication because the connection appears to both the Launchpad and the
  computer as a serial port, even though it may actually use
  a USB cable. Bytes are sent one after another (serially) from the Launchpad
  to the computer.

  You can use the Energia serial monitor to view the sent data, or it can
  be read by Processing or another program capable of reading 
  data from a serial port. The Processing code below graphs the data received 
  so you can see the value of the analog input changing over time.

  The circuit:
  Any analog input sensor can be attached to an analog pin.  
  For this example, a potentiometer is attached to analog pin 3. 

  created 2006
  by David A. Mellis
  modified 9 Apr 2012
  by Tom Igoe and Scott Fitzgerald
  Modified 15 April 2013
  By Sean Alvarado

  Hardware Required:
  * MSP-EXP430G2 LaunchPad
  * Potentiometer
  
  This example code is in the public domain.

*/
#define NUMBER 32 // take number / 2
int ledState = HIGH;
uint8_t i = 0;
uint32_t average = 0;
uint32_t values[NUMBER];
uint8_t j = 0;
boolean flag = false;
boolean running = false;
int debounceCounter = 0;
// From http://forum.43oh.com/topic/4730-using-the-internal-temp-sensor-on-f5529/
static const uint16_t *ADC15TEMP30 = (uint16_t *)0x1A1A;
static const uint16_t *ADC15TEMP85 = (uint16_t *)0x1A1C;
static const uint16_t *ADC25TEMP30 = (uint16_t *)0x1A22;
static const uint16_t *ADC25TEMP85 = (uint16_t *)0x1A24;
void setup() {
    pinMode(RED_LED, OUTPUT);
  pinMode(GREEN_LED, OUTPUT);
//  analogReference(INTERNAL1V5);
  analogReference(INTERNAL2V5);
  analogRead(TEMPSENSOR); // first reading usually wrong
  // initialize the serial communication:
  Serial.begin(9600);
  pinMode(PUSH2, INPUT_PULLUP);
  digitalWrite(RED_LED, HIGH);
  digitalWrite(GREEN_LED, LOW);
  
  for (j=0; j<NUMBER; j++) {
    values[j]=0;
  }
  average = 0;
  j=0;  
}

void printDec(uint32_t ui) {
  if (ui < 100){
    // ensure that a leading zero is printed so that 
    // all temperatures will have 4 digits
    Serial.print("0");
  }
  if (ui < 999){
    Serial.print(ui/10, DEC);
    Serial.print(".");
    Serial.print(ui%10, DEC);
  } 
  else {
    Serial.print("99.9");
  }
}



// From http://forum.43oh.com/topic/4730-using-the-internal-temp-sensor-on-f5529/
int16_t GetTempX10(int oversampling)
{
  uint32_t reading=0;
  int16_t result = 0;
  
  analogReference(INTERNAL1V5); //use the 1.5V internal reference
  
  //for(int i=0; i<(1<<oversampling); i++)
  //  reading+=analogRead(TEMPSENSOR); //do 2^oversampling reads
  //reading>>=oversampling; //and divide by 2^oversampling, we are left with the average from 2^oversampling reads.
  reading = analogRead(TEMPSENSOR);
  /* Celsius calculations:
  return map(reading,*ADC15TEMP30,*ADC15TEMP85,300,850); //convert our reading to temp x 10 & return
  */
  /* Farenheight calculations: */
//  result = map(reading,*ADC15TEMP30,*ADC15TEMP85,860,1850); //convert our reading to temp x 10 & return
  result = map(reading,*ADC25TEMP30,*ADC25TEMP85,860,1850); //convert our reading to temp x 10 & return
  
  return (result);
}
void loop() {
    if (running){
    ledState = !ledState;
    // LEDs: green = ready; red = acquisition
    digitalWrite(RED_LED, ledState);
    
   // Formula: http://www.43oh.com/forum/viewtopic.php?p=18248#p18248
    average -= values[j];
    values[j] = (uint32_t)GetTempX10(1);  //2^1 oversampling
    average += values[j];
    
    if (flag) {

      printDec(average/NUMBER);
      Serial.print("\n");
      //flag = false;

    }
    j++;
    if (j==NUMBER) {
      flag=true;
      j = 0;
    } 
  } 
  if (digitalRead(PUSH2)==LOW) {
    if (debounceCounter == 2){
      running = !running;
      digitalWrite(GREEN_LED, running);
      digitalWrite(RED_LED, HIGH);
      debounceCounter = 9;
    } else {
      debounceCounter++;
    }
  } else {
    debounceCounter = 0;
  }
  delay(40);
}

/* 
// Processing code for this example

// Graphing sketch


// This program takes ASCII-encoded strings
// from the serial port at 9600 baud and graphs them. It expects values in the
// range 0 to 1023, followed by a newline, or newline and carriage return

// Created 20 Apr 2005
// Updated 18 Jan 2008
// by Tom Igoe
// modified 16 April 2013
// by Sean Alvarado
// This example code is in the public domain.

import processing.serial.*;

Serial myPort; // The serial port
int xPos = 1; // horizontal position of the graph

void setup () {
  // set the window size:
  size(400, 300); 

  // List all the available serial ports
  println(Serial.list());
  // Open whatever port is the one you're using for the Launchpad.
  myPort = new Serial(this, Serial.list()[0], 9600);
  // don't generate a serialEvent() unless you get a newline character:
  myPort.bufferUntil('\n');
  // set inital background:
  background(0);
}
void draw () {
  // everything happens in the serialEvent()
}

void serialEvent (Serial myPort) {
  // get the ASCII string:
  String inString = myPort.readStringUntil('\n');

  if (inString != null) {
    // trim off any whitespace:
    inString = trim(inString);
    // convert to an int and map to the screen height:
    float inByte = float(inString); 
    inByte = map(inByte, 0, 1023, 0, height);

    // draw the line:
    stroke(127,34,255);
    line(xPos, height, xPos, height - inByte);

    // at the edge of the screen, go back to the beginning:
    if (xPos >= width) {
      xPos = 0;
      background(0); 
    } 
    else {
      // increment the horizontal position:
      xPos++;
    }
  }
}
*/
