/*
* Copyright (c) 2014, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
  *
* *  Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
* *  Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* */

// Constructor for CcsDebugLoader objects
window.CcsDebugLoader = function() {
  console.log('Constructor for CcsDebugLoader objects called... ');
  return this.init();
}

var hacksEnabled = !(DS.StartCloudAgentNodeWebkit);
var hacksToDisableCCSCloudStuff = true;
var hacksEnabledForRemotejs = false;

// Define the ccs_debug_loader object's methods
CcsDebugLoader.prototype = {
  init: function() {
    console.log('CcsDebugLoader.init called... ');
    this.localServer.init = this.localServer._init.bind(this);
    this.localServer.init();
  },

  localServer: {

    console: {

      interval: null,

      init: function() {


        console.log('CcsDebugLoader.localServer.console.init called... ');

      },
      show: function() {
        if (!hacksToDisableCCSCloudStuff) {
          m_terminalView.show(m_terminalView.DEBUG);
        }
      },
      append: function(text) {
        if (!hacksToDisableCCSCloudStuff) {
          m_terminalView.append(m_terminalView.DEBUG, text);
        } else {
          console.log(text);
        }
      },
      beginProgress: function(text) {
        if (!hacksToDisableCCSCloudStuff) {

          var _self = this;
          if (_self.interval == null) {
            _self.show();
            _self.append("\n" + text);
            _self.interval = setInterval(function() {
              _self.append(".");
            }, 1000);
          }
        }
      },
      endProgress: function(text) {
        var _self = this;
        if (!hacksToDisableCCSCloudStuff) {
          _self.append("\n" + text);
          if (_self.interval) {
            clearInterval(_self.interval);
            _self.interval = null;
          }
        }
      }
    },

    localServerInfo: {
      port: null,
      version: "0"
    },

    portFailure: null,
    portRequested: false,
    ds: null,
    getDebugServer: function() {
      return this.ds;
    },
    mainSocket: undefined,
    getMainSocket: function() {
      return (this.mainSocket);
    },
    _init: function() {
      console.log('CcsDebugLoader.localServer.init called... ');
      //bc: Ensure that 'this' in the following functions is bound to the backplane object:
      // note that 'this' at this point is the object that called the init function.
      this.localServer.reset = this.localServer._reset.bind(this);
      this.localServer.resetNumRetries = this.localServer._resetNumRetries.bind(this);
      this.localServer.setIsSecureWebPage = this.localServer._setIsSecureWebPage.bind(this.localServer);
      this.localServer.onGetDSError = this.localServer._onGetDSError.bind(this);
      this.localServer.startTICloudAgent = this.localServer._startTICloudAgent.bind(this);
      this.localServer.getDS = this.localServer._getDS.bind(this);
      this.localServer.retryCallback = this.localServer._retryCallback.bind(this);

      if (!hacksToDisableCCSCloudStuff) {
        ide.addEventListener("socketMessage", function(e) {
          console.log('socketMessage eventListener called... ');

          if ("ccsproxy" == e.message.type && "setLocalServerPort" == e.message.id) {
            this.localServerInfo.port = e.message.port;
            this.localServerInfo.version = e.message.version;
          }
        });
      } else {
        /* TODO:  Q: How do the ide and ccsproxy objects map to resource explorer? */
      }
      this.localServer.console.init();
    },
    init: undefined,

    getWikiLink: function(part) {
      var a = part ? part : "";
      return '<br>For more information, please visit <a href="' +
        "http://processors.wiki.ti.com/index.php/TI_Cloud_Agent_Troubleshooting_Guide" + a +
        '" target="_blank">TI Cloud Agent wiki page</a>.';
    },

    _onGetDSError: function(error) {
      // due to binding, 'this' refers to the ds object itself.
      var _self = this;
      this.localServer.reset();
      if (error instanceof DS.MissingSSLCertificate) {
        var textToShow = "Unable to open connection to TI Cloud Agent.\n" +
          "This is most likely caused by a missing SSL certificate." +
          _self.getWikiLink("#Local_Host_SSL_Certificate_Installation") + "\n";
        this.localServer.console.endProgress(textToShow);
        return (textToShow);
      }

      this.localServer.console.endProgress(error);

      var button = "<button type=\"button\" onclick=\"ticloudagent_downloadLocalServer()\">Download</button>";
      var text = error.name.indexOf("version") != -1 ?
      " Please " + button + " and install a newer version." :
      "\nCould not communicate with driver, if you have not already done so, please " + button +
      " and install TI Cloud Agent.<br>If you have already installed it, try executing the operation again." +
      this.getWikiLink();
      _self.console.append(text);
      return (text);
    },
    onGetDSError: undefined,
    _reset: function() {
      var _self = this;
      this.localServerInfo = {
        port: null,
        version: "0"
      };
      this.localServer.portFailure = null;
      this.localServer.portRequested = false;
      this.localServer.ds = null;
      this.localServer.nullServerPort();

    },
    reset: undefined,
    _resetNumRetries: function() {
      this.numRetries = 0;
    },
    resetNumRetries: undefined,
    nullServerPort: function(port) {
      var _self = this;
      try {
        var path = ticloudagent_portAPI("setLocalServerPort") + "?port=null";
        var req = new XMLHttpRequest();
        req.open("GET", path, true);
        req.send();
      }
      catch (ex) {
      }
    },
    numRetries: 0,
    maxNumRetries: 5,
    _setIsSecureWebPage: function(value) {
      this.isSecureWebPage = value;
    },
    setIsSecureWebPage: undefined,
    isSecureWebPage: false,
    _getDS: function(callback, timeToWaitBeforeCallingDSInit, numRetries) {
      // see localserver.init function for binding
      var _self = this.localServer;
      if (_self.ds) {
        console.log('getDS: _self.ds is not null, so returning.');
        return callback(null);
      }

      _self.console.beginProgress("\nStarting TI Cloud Agent.");
      console.log('CcsDebugLoader.startTICloudAgent called from node-webkit app... ');
      // only will get here if application is being run by node-webkit
      var fullUrl = "";
      try {
        if ((window.location.pathname !== undefined) && (window.location.pathname !== null)) {
          fullUrl = window.location.pathname;
        }
      }
      catch (ex) {
        console.log('Exception setting window.location.pathname: ' + ex.message);
      }

      //TODO: refactor duplicated code in the if / then / else statements below
      if (DS.StartCloudAgentNodeWebkit) {
        if ((fullUrl.indexOf("/") === 0) && (fullUrl.indexOf(":/") > 0)){
          // get rid of the leading slash in windows pathnames (e.g. /C:/...)
          fullUrl = fullUrl.substring(1, fullUrl.length);
        }
        var indexOfRootDir = fullUrl.lastIndexOf('app');
        if (indexOfRootDir < 0) {
          // see if it is being run from one of the component demos
          indexOfRootDir = fullUrl.indexOf('components');
        }
        if (indexOfRootDir >= 0) {
          var APP_ROOT = fullUrl.substring(0, indexOfRootDir);

          DS.StartCloudAgentNodeWebkit({appRoot: APP_ROOT}, function(regParams) {


            var doneCallback = function(error, ds) {
              if (error) {
                if (error instanceof  DS.InvalidVersion) {
                  console.log('getDS: getLocalServerInfo - error = bad version of TI Cloud Agent');
                  return callback("The version of TI Cloud Agent on this computer is old.");
                }
                if (error instanceof DS.MissingSSLCertificate) {
                  return callback(error);
                }
              }

              _self.ds = ds;
              callback(null);
            };
            var initFailedCallback = function(error) {
              console.log('getDS: initFailedCallback: error = ' + error.msg);
              return callback(error);
            };
            var terminateCallback = function() {
              _self.console.endProgress("\nThe connection to TI Cloud Agent closed.");
              _self.reset();
            };

            var initParams = {
              port: regParams.port,
              version: regParams.version,
              isSecureWebPage: _self.isSecureWebPage
            };


            var socketOpenedOk = false;
            if (_self._getDsCompleteCallback === undefined) {
              _self._getDsCompleteCallback = callback;
            }
            var tmpTimer = window.setTimeout(function() {
              _self.numRetries++;
              console.log('getDS: about to call DS.Init... (attempt ' + _self.numRetries + ')');
              if (_self.numRetries < _self.maxNumRetries) {
                DS.Init(initParams, doneCallback, initFailedCallback, terminateCallback, _self.retryCallback, _self);
              } else {
                var error = {msg: 'Failed to connect to webSocket after ' + _self.numRetries + ' attempts.'};
                _self.numRetries = 0;
                initFailedCallback(error);
              }
            }, timeToWaitBeforeCallingDSInit); // give things time to settle (e.g. for ports to close after page
                                               // reloaded) before calling getDS and opening the websocket


          });
        }
      } else {
        _self.getLocalServerInfo(function(error, info) {
          if (error) {
            console.log('getDS: getLocalServerInfo - error = ' + error);
            return callback(error);
          }
          ;
          var doneCallback = function(error, ds) {
            if (error) {
              if (error instanceof  DS.InvalidVersion) {
                console.log('getDS: getLocalServerInfo - error = bad version of TI Cloud Agent');
                return callback("The version of TI Cloud Agent on this computer is old.");
              }
              if (error instanceof DS.MissingSSLCertificate) {
                return callback(error);
              }
            }

            _self.ds = ds;
            callback(null);
          };
          var initFailedCallback = function(error) {
            console.log('getDS: initFailedCallback: error = ' + error.msg);
            return callback(error);
          };
          var terminateCallback = function() {
            _self.console.endProgress("\nThe connection to TI Cloud Agent closed.");
            _self.reset();
          };
          if (hacksEnabled) {
            info.port = 2222;
            info.version = ".2";
          }
          console.log('getDS: port =' + info.port + ', version = ' + info.version + ', isSecureWabPage=' + _self.isSecureWebPage);
          var initParams = {
            port: info.port,
            version: info.version,
            isSecureWebPage: _self.isSecureWebPage

          };


          var socketOpenedOk = false;
          if (_self._getDsCompleteCallback === undefined) {
            _self._getDsCompleteCallback = callback;
          }
          var tmpTimer = window.setTimeout(function() {
            _self.numRetries++;
            console.log('getDS: about to call DS.Init... (attempt ' + _self.numRetries + ')');
            if (_self.numRetries < _self.maxNumRetries) {
              DS.Init(initParams, doneCallback, initFailedCallback, terminateCallback, _self.retryCallback, _self);
            } else {
              var error = {msg: 'Failed to connect to webSocket after ' + _self.numRetries + ' attempts.'};
              _self.numRetries = 0;
              initFailedCallback(error);
            }
          }, timeToWaitBeforeCallingDSInit); // give things time to settle (e.g. for ports to close after page reloaded)
                                             // before calling getDS and opening the websocket

        });
      }
    },
    getDS: undefined,
    _getDsCompleteCallback: undefined,
    _retryCallback: function() {
      this.localServer.getDS(this.localServer._getDsCompleteCallback, 5000);
    },
    retryCallback: undefined,

    getLocalServerInfo: function(callback) {
      var _self = this;
      console.log('CcsDebugLoader.getLocalServerInfo called... ');

      if (hacksEnabled) {
        /* HACK until Resource Explorer supports port resolution  */
        _self.localServerInfo.port = 2222;
        _self.localServerInfo.version = '.2';

      }

      if (_self.localServerInfo.port || _self.portFailure) {
        return callback(_self.portFailure, _self.localServerInfo);
      }
      if (!_self.portRequested) {
        _self.portRequested = true;
        var path = ticloudagent_portAPI("getLocalServerPort");
        console.log("port requested. path = " + path);
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
          console.log("in req.onreadystatechange.  readyState=" + req.readyState + ", req.status=" + req.status);
          if (req.readyState == 4 && req.status == 200) {
            if (req.responseText) {
              _self.localServerInfo = JSON.parse(req.responseText);
            }
            console.log("in req.onreadystatechange.  localServerInfo.port =" + _self.localServerInfo.port);
            if (!_self.localServerInfo.port) {
              _self.startTICloudAgent();
            }
          }
          /* if (hacksToDisableCCSCloudStuff) {
           if (req.readyState == 4 && req.status == 200) {

           _self.startCCStools();
           }
           }
           */
        };
        req.open("GET", path, true);
        req.send();
      }
      if (!hacksEnabled) {
        _self.waitForPort(callback);
      } else {
        _self.portFailure = null;
        callback(_self.portFailure, _self.localServerInfo);
      }

    },

    _startTICloudAgent: function() {  // was startCCStools
      var _self = this;
      if (!DS.StartCloudAgentNodeWebkit) {

        console.log('CcsDebugLoader.startTICloudAgent called from browser... ');
        var hostPart = window.location.hostname;
        var portPart = location.port ? ":" + location.port : "";

        var apiPart = ticloudagent_portAPI("setLocalServerPort");

        if (hacksEnabledForRemotejs) {
          apiPart = ccstools_portAPI("foo");
        }
        var api = "https://" + hostPart + portPart + apiPart;
        var sessionID = "none";
        var defaultPort = "2222";
        var allParams = "&sessionID=" + sessionID + "&wsPort=" + defaultPort + "&registrationAPI=" + api;
        var newHref = "ticloudagent://host/path?module=debug" + allParams;
        console.log('URL = ' + newHref);
        this.localServer.startURLHandler(newHref);
      }
    },
    startTICloudAgent: undefined,
    startURLHandler: function(uri) {
      if (ticloudagent_isFirefox() || ticloudagent_isSafari()) {
        var iframe = document.createElement('iframe');
        if (ticloudagent_isFirefox()) {
          iframe.style.display = "none";
        }
        document.body.appendChild(iframe);
        try {
          iframe.contentWindow.location.href = uri;
        } catch (ex) {
        }
        if (ticloudagent_isFirefox()) {
          // On v22.0 and earlier of Firefox, iframe.remove is not defined.
          // See http://compatibility.shwups-cms.ch/en/home?&property=__createdObjects.iframe.removeNode
          try {
            if (iframe.remove) {
              iframe.remove();
            } else {
              if (iframe.removeNode) {
                iframe.src = "";  // see
                                  // http://itknowledgeexchange.techtarget.com/itanswers/iframeremovenode-only-hides-does-not-destroy-the-iframe-node/
                iframe.removeNode(true);
              }
            }
          } catch (ex1) {
            console.log('ccs_debug_loader_bc: iframe could not be removed');
          }
        }
        return;
      }
      var isInIframe = (window.location !== window.parent.location);
      if (isInIframe) {
        window.parent.location.href = uri;
      } else {
        window.location.href = uri;
      }
    },

    waitForPort: function(callback, failTime) {
      var TIMEOUT = 12000;
      var _self = this;
      var startTime = new Date().getTime();
      var interval = setInterval(function() {
        var now = new Date().getTime();
        if (hacksEnabled) {
          if (!_self.portFailure && now - startTime > TIMEOUT) {
            clearInterval(interval);
            callback(_self.portFailure, _self.localServerInfo);
          }
        } else {
          if (!_self.portFailure && now - startTime > TIMEOUT) {
            _self.portFailure = "Timeout connectiong to TI Cloud Agent.";
          }
        }
        if (_self.localServerInfo.port || _self.portFailure) {
          clearInterval(interval);
          callback(_self.portFailure, _self.localServerInfo);
        }
        ;
      }, 500);
    }
  },

  downloadFile: function(target) {
    var _self = this;
    var path = target.getAttribute("path");
    var name = target.getAttribute("name");
    if (ticloudagent_isSafari()) {
      var a = document.createElement("a");
      a.download = name;
      a.href = path;
      a.click();
      delete a;
    }
    else {
      var req = new XMLHttpRequest();
      req.open('GET', path, true);
      req.responseType = "blob";
      req.onreadystatechange = function() {
        if (req.readyState == 4) {
          require("./FileSaver");
          saveAs(req.response, name);
        }
      };
      req.send(null);
    }
  }

  /* bc: remove stuff related specifically to cloud9

   });
   });
   */
};
/* end of prototype */

function ticloudagent_portAPI(name) {
  if (hacksEnabledForRemotejs) {
    var api = "/foo";
  } else {
    var path = window.location.pathname;
    var path = path.substring(0, path.indexOf('/ti-core-backplane/'));
    console.log('ticloudagent_portAPI: path =[' + path + ']');
    var pathEndsWithSlash = path.indexOf("/", path.length - 1) !== -1;
    var glueSlash = pathEndsWithSlash ? "" : "/";
    var api = path + glueSlash + "api/" + name;
  }
  return api;
};

function ticloudagent_downloadLocalServer() {
  //var path = ticloudagent_portAPI("getLocalServerFile")+"?os=" + ticloudagent_getOS();
  var name = ticloudagent_getAppName();
  var path = '../TICloudAgent/' + name;

  if (ticloudagent_isSafari()) {
    var a = document.createElement("a");
    a.download = name;
    a.href = path;
    a.click();
    delete a;
  }
  else {
    var req = new XMLHttpRequest();
    req.open('GET', path, true);
    req.responseType = "blob";
    req.onreadystatechange = function() {
      if (req.readyState == 4) {
        saveAs(req.response, name);
      }
    };
    req.send(null);
  }
  ;
};

function ticloudagent_getAppName() {
  if (navigator.appVersion.indexOf("Win") != -1) {
    return "ticloudagent-0.2-windows-installer.exe";
  }
  if (navigator.appVersion.indexOf("Mac") != -1) {
    return "ticloudagent-0.2-osx-installer.dmg";
  }
  return "ticloudagent-0.2-linux-installer.run";
};

function ticloudagent_getOS() {
  if (navigator.appVersion.indexOf("Win") != -1) {
    return "win64";
  }
  if (navigator.appVersion.indexOf("Mac") != -1) {
    return "mac64";
  }
  return "linux64";
};

function ticloudagent_isChrome() {
  return navigator.userAgent.indexOf("Chrome") !== -1;
};

function ticloudagent_isSafari() {
  return navigator.userAgent.indexOf("Safari") !== -1 && navigator.userAgent.indexOf("Chrome") == -1;
};

function ticloudagent_isFirefox() {
  return navigator.userAgent.indexOf("Firefox") !== -1;
};


